% Specification that discretizes all continuous variables

model = struct();
priors = struct();

% Load model that we will simplify 
model_file_nyrt_fix_time_uh

% Filepath
model.model_name = 'nyrt_fix_time_uh_overfit';
model.gen_script = mfilename('fullpath'); % name of script that generated model file (i.e. name of this script)
model.gen_date = datetime('today'); % date model file was generated

% Model structure
model.donor_uh = true;

% List of history-specific patient characteristics
model.offer_chars = {'match_date','geo','a_mm','b_mm','dr_mm','c_abo'};    % List of match-specific characteristics from offer file

model.newvars = {{{model.time_var{1},model.time_var{2}},@(x) (x{2}-x{1})/365,'wait_time'},...
                 {'wait_time', @(x) log(min(max(x,1/365),7)), 'log_wait_time'}, ...              
                 {'a_mm',@(x) x==2,'amis_2'},... 
                 {'b_mm',@(x) x==2,'bmis_2'},... 
                 {'dr_mm',@(x) x==2,'drmis_2'},... 
                 {'a_mm',@(x) x==1,'amis_1'},... 
                 {'b_mm',@(x) x==1,'bmis_1'},... 
                 {'dr_mm',@(x) x==1,'drmis_1'},... 
                 {{'a_mm','b_mm','dr_mm'},@(x) (x{1} + x{2} + x{3})==0,'zero_mm'},... 
                 {'c_abo',@(x) x~=2,'abo_compatible'},... % 2 = abo identical
                 {'abo',@(x) any([x==1 x==11 x==12],2),'pt_abo_A'},... 
                 {'abo',@(x) x==0,'pt_abo_O'},... 
                 {'opo_dsa_id',@(x) x==model.local_opo, 'nyrt_donor'},...
                 {{'opo_region_id','opo_dsa_id'},@(x) (x{1}==9).*(x{2}~=model.local_opo), 'regional_donor'},...
                 {'opo_region_id',@(x) x~=9, 'national_donor'},...
                 {{'geo','opo_dsa_id'},@(x) (x{1}==1).*(x{2}~=model.local_opo),'non_nyrt_local_match'},...
                 {'hist_hypertens_don',@(x) x==1,'donor_hypertension'}, ...
                 {'num_prev_tx',@(x) x>0,'prior_transplant'},...
                 {{'init_date','dialysis_date'},@(x) x{1} >= x{2},'on_dialysis_at_init_date'},...
                 {{'init_date','dialysis_date'},@(x) (x{2}>0).*log(min(max(x{1}-x{2},1),5000)/365),'log_dialysis_time_at_init'},... % Conditioning on non-missing dialysis
		 {{'init_date','dialysis_date'},@(x) (x{2}>0).*(x{1}-x{2})/365,'dialysis_time_at_init'},... % Conditioning on non-missing dialysis
                 {'cpra',@(x) x>=0.8, 'cpra_gt80'},...; 
                 {'cpra',@(x) x>=0.2, 'cpra_gt20'},...; 
                 {'cpra',@(x) x==0,   'cpra_0'}}; 

model.fixed.linear = {'pt_diabetes','cpra_gt80','cpra_gt20','cpra_0', ...
		      'prior_transplant','don_cod_anoxia','don_cod_stroke', 'don_cod_cns', ...
		      'ecd_donor','dcd', 'don_male','donor_hypertension', ...
		      'don_creat_p5_1p0', 'don_creat_1p0_1p5','don_creat_ge_1p5',...		    
		      'zero_mm', 'amis_2', 'bmis_2', 'drmis_2', 'amis_1', 'bmis_1', 'drmis_1',...
		      'abo_compatible','regional_donor','national_donor', ...
		      'non_nyrt_local_match', 'pt_abo_A', 'pt_abo_O', ...
		      'on_dialysis_at_init_date'};,

model.fixed.pwlin = {{'init_age',10:10:70},{'age_don',10:10:70},...
		     {'wait_time',[1:10 12]},{'dialysis_time_at_init',[1:6 8 10]},...
		     {'end_bmi_calc',18:2:32},{'tot_serum_album',3.5:0.2:4.5},...
		     {'cpra',[0.2 0.8 0.9 0.95 0.98]}};

pt_age_vars = {'init_age','init_age_gt_10','init_age_gt_20','init_age_gt_30',...
	       'init_age_gt_40','init_age_gt_50','init_age_gt_60','init_age_gt_70'};
cpra_vars = {'cpra_gt_0.2','cpra_gt_0.8','cpra_gt_0.9','cpra_gt_0.95'};
wait_time_vars = {'wait_time','wait_time_gt_2','wait_time_gt_4','wait_time_gt_6',...
		  'wait_time_gt_8','wait_time_gt_10','wait_time_gt_12'};
dialysis_time_vars = {'dialysis_time_at_init','dialysis_time_at_init_gt_2',...
		      'dialysis_time_at_init_gt_4','dialysis_time_at_init_gt_6',...
		      'dialysis_time_at_init_gt_8','dialysis_time_at_init_gt_10'};
pt_vars = [pt_age_vars,cpra_vars,wait_time_vars,dialysis_time_vars];

don_age_vars = {'age_don','age_don_gt_10','age_don_gt_20','age_don_gt_30',...
		'age_don_gt_40','age_don_gt_50','age_don_gt_60'};

other_vars = {'prior_transplant','pt_diabetes','pt_abo_A','pt_abo_O',...
	      'ecd_donor','dcd','nyrt_donor','abo_compatible'};

model.fixed.interactions ={{'zero_mm','ecd_donor'},...
                           {'zero_mm','dcd'}};

dummies = {'zero_mm','nyrt_donor','prior_transplant','abo_compatible'};
with_dummies = [don_age_vars,pt_age_vars,wait_time_vars,dialysis_time_vars,cpra_vars];
for jj = 1:length(dummies)
	   for ii = 1:length(with_dummies)
		      model.fixed.interactions{end+1} = {dummies{jj},with_dummies{ii}};
           end
end
for jj = 1:length(don_age_vars)
	   for ii = 1:length(pt_vars)
		      model.fixed.interactions{end+1} = {don_age_vars{jj},pt_vars{ii}};
           end
end
with_wait_time = cpra_vars;
for jj = 1:length(wait_time_vars)
	   for ii = 1:length(with_wait_time)
		      model.fixed.interactions{end+1} = {wait_time_vars{jj},with_wait_time{ii}};
	   end	      
end

% Store model in /structural_model/input/ (../input/)
save([models_folder model.model_name],'model','priors');
