// Determine which offers are relevant for CCP estimation and for cutoffs in each match run
// Also merges in match run numbers from UNOS and generates a unique organ_sequence_num for a donor's offers

// Generated variables:

//  - n_ki_available_match: # kidneys available at the beginning of each match run
//        - Anything placed in the offer sequence was available initially
//        - If the last placement of an organ (across match runs) was in an earlier match run, it's not available in later match runs
//        - Non-local match runs that trace back to a local placement only have the trace-back organ(s) available (e.g. just the left kidney)
//     - Used to compute ro_cutoff (and hence real_offer). Not needed for other variables.

//  - final_decision: whether patient eventually accepted a kidney from the donor. Algorithm:
//        - Find last acceptance of each kidney for each donor (by match_run_num)
//        - Set final_decision of each patient to the maximum of the last acceptance indicator across offers and match runs

//  - ro_*:
//      - ro_cutoff: was offer above bin+points cutoff in the match run. ro_cutoff==0 after the last available organ has been placed in the match run
//      - ro_duplicates: de-duplicates within match run. For each patient, ro_duplicates==0 for later offers in the match run
//      - ro_matchrun: if a patient is in multiple match runs, set ro_matchrun==0 for earlier match runs
//      - ro_response: c_accept=="Yes"|"No". Patient actually said yes or no

//  - real_offer: whether an offer should be counted for CCP estimation (product of all ro_* variables)


cap drop match_run_num  // generated in classify_match_runs.ado
cap drop organ_sequence_num

// save current offer dataset, keep relevant variables
tempfile hold_offers
save `hold_offers', replace

keep donor_id-wl_id_code refusal_id descrip kidney_points offer_accept organ_placed *_disposition *opo* *listing* c_accept

// construct basic info about placements by match run
gen l_placed = (organ_placed == "L" | organ_placed == "B")
gen r_placed = (organ_placed == "R" | organ_placed == "B")
bys donor_id match_id_code (ptr_row_order pb_offer_seq): gen match_run_mark = _n==1
by donor_id match_id_code: egen n_placements_match = total(l_placed + r_placed)


// merge in match run numbers from UNOS
preserve
use `primary_files'match_run_num_optn, clear
cap rename MATCH_RUN_NUM match_run_num
cap rename MATCH_ID_CODE match_id_code
save `primary_files'match_run_num_optn, replace
restore
merge m:1 match_id_code using `primary_files'match_run_num_optn, keep(master match) nogen
codebook match_run_num

// generate organ_sequence_num, a unique ordering of offers for each donor
bys donor_id (match_run_num ptr_row_order pb_offer_seq): gen organ_sequence_num = _n
isid donor_id organ_sequence_num

// de-duplicate offers involving the same donor-patient pair, within and across match runs
bys donor_id wl_id_code (match_run_num ptr_row_order): gen max_pt_match_run_num = match_run_num[_N]
gen byte ro_matchrun = match_run_num == max_pt_match_run_num  // counts offers from last match run where patient appeared
gen byte ro_response = c_accept == "Yes" | c_accept == "No"  // counts offers we consider a genuine "yes" or "no"
tab refusal_id c_accept 
by donor_id wl_id_code match_run_num: gen byte ro_duplicates = _n==1  // counts first offer to a patient in each match run
    
// final decision for each patient
bys donor_id l_placed (organ_sequence_num): gen last_l_placement = l_placed*(_n==_N)  // marks donor's last l placement in offer sequence
bys donor_id r_placed (organ_sequence_num): gen last_r_placement = r_placed*(_n==_N)  // marks donor's last r placement in offer sequence
bys donor_id wl_id_code: egen final_decision = max(last_l_placement | last_r_placement)  // marks whether each PATIENT made a final acceptance

    
// mark whether each offer made the bin/points cutoff for the match run
    // key question: for match runs with only one placement, were both kidneys available or just one?
    // to figure this out, identify reasons why a kidney might have previously been taken out of circulation before each match run
    //    1. only one kidney was offered from the start, either due to a KP transplant or the KI dispositions
    //    2. the match run is run by a non-recovering OPO, and was generated by an earlier local match run that placed ONE kidney to a patient from the non-recovering OPO
    //    3. one kidney's final acceptance was in an earlier match run

// 1.1 look for KP transplants
preserve
use "~/organs-raw/raw-extract-3/STAR/Kidney_ Pancreas_ Kidney-Pancreas/KIDPAN_DATA.DTA", clear
keep if organ=="KP" & don_ty == "C"
keep donor_id
isid donor_id
tempfile kp_transplants
save `kp_transplants', replace
restore
merge m:1 donor_id using `kp_transplants', keep(master match)
gen kp_tx = _merge == 3
drop _merge

// 1.2. identify donors where only one kidney was offered from the start, either due to a KP transplant or the KI dispositions
gen one_ki_init = kp_tx | ((kil_disposition < 5 | kir_disposition < 5) & kib_disposition != 5 & kib_disposition != 6)

// 2. identify match runs following the final placement of a kidney
bys donor_id: egen match_run_last_l_pl = max(match_run_num*last_l_placement)
by donor_id: egen match_run_last_r_pl = max(match_run_num*last_r_placement)
gen one_ki_left_match_run = match_run_num > min(match_run_last_l_pl,match_run_last_r_pl)

tempfile hold_on
save `hold_on', replace

// 3. identify non-local match runs tracing back to local placements

  // local placements
  preserve
  keep if local_match == 1 & organ_placed != ""
  gen placed_opo = listing_ctr_dsa_id if payback == 0
  replace placed_opo = payback_opo if payback == 1
  keep donor_id wl_id_code placed_opo donor_opo organ_placed payback
  tempfile local_placements
  save `local_placements', replace
  restore

  // non-local match runs
  keep if local_match == 0 & match_run_mark == 1
  keep donor_id match_id_code match_opo
  tempfile non_local_match_runs
  save `non_local_match_runs', replace

  // match all non-local match runs to all local placements, mark cases where non-local match run opo got a local placement for that donor
  joinby donor_id using `local_placements'
  gen trace_back = match_opo == placed_opo
  tab trace_back payback
  keep if trace_back == 1  // keep only non-local matches that can be traced to local placements
  keep donor_id match_id_code trace_back organ_placed match_opo
  bys donor_id match_id_code organ_placed: keep if _n==1
  isid donor_id match_id_code organ_placed
  bys donor_id match_id_code: egen n_ki_trace_back = total((organ_placed=="L") + (organ_placed=="R") + 2*(organ_placed=="B"))
  replace n_ki_trace_back = 2 if n_ki_trace_back > 2
  tab n_ki_trace_back, m
  bys donor_id match_id_code: keep if _n==1  // de-duplicate
  drop match_opo organ_placed
  merge 1:m donor_id match_id_code using `hold_on', assert(match using) nogen

// variable for whether there was one kidney available at the beginning of a match run AND there was one placement in the match run
gen n_ki_available_match = 2 - (n_ki_trace_back==1 | one_ki_left_match_run==1 | one_ki_init==1)
gen one_ki_one_placement = n_placements_match==1 & n_ki_available_match==1
replace one_ki_one_placement = 0 if n_ki_trace_back==2

// finally: determine relevant cutoff in each match run
bys donor_id match_run_num: egen last_placement_match_run = max(organ_sequence_num*(organ_placed != ""))
gen after_last_placement_match = organ_sequence_num > last_placement_match_run
gen byte ro_cutoff = 1
replace ro_cutoff = 0 if n_placements_match==2 & after_last_placement_match==1
replace ro_cutoff = 0 if one_ki_one_placement==1 & after_last_placement_match==1

// whether to include offer in CCP analysis
gen real_offer = ro_cutoff*ro_duplicates*ro_matchrun*ro_response

// merge back to full offer dataset
keep donor_id match_id_code wl_id_code ptr_row_order pb_offer_seq match_run_num organ_sequence_num ro_* real_offer final_decision n_ki_available_match
merge 1:1 donor_id match_id_code ptr_row_order pb_offer_seq using `hold_offers', assert(match) nogen

save `primary_files'ptr_offers, replace // save an intermediate dataset to be used in document_sample_selection.do 
